Imports System.Web.UI
Imports System.Collections.Generic
Imports System.Reflection

Imports DotNetNuke
Imports DotNetNuke.Services.Exceptions
Imports DotNetNuke.Services.Localization
Imports System.Drawing
Imports System.IO

Namespace WESNet.DNN.Modules.ImageEditor

    Partial Class ImageEditorDemo
        Inherits Entities.Modules.PortalModuleBase

        Private _FileList As ArrayList
        Private _ThumbSuffix As String = "_th"
        Private _Folder As String = "/"

        Protected Sub Page_Init(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Init
            If Framework.AJAX.IsInstalled Then
                Framework.AJAX.RegisterScriptManager()
                ScriptManager.GetCurrent(Page).RegisterPostBackControl(btnLoad)
            End If
        End Sub

        Protected Sub Page_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Load
            FileUpload1.Attributes.Add("onchange", "fileUpload_OnChange(this,event,'" & btnLoad.ClientID & "');")
            If Not IsPostBack Then
                With Me.ImageEditor1
                    .CachePersistence = CType([Enum].Parse(GetType(WESNet.AJAX.Controls.ImageEditor.CachePersistenceType), rblCaching.SelectedValue), WESNet.AJAX.Controls.ImageEditor.CachePersistenceType)
                    .ToolbarOrientation = CType([Enum].Parse(GetType(System.Web.UI.WebControls.Orientation), rblOrientation.SelectedValue), System.Web.UI.WebControls.Orientation)
                    .Rescale = cbRescale.Checked
                    '.UseWebService = cbUseWebService.Checked
                    .Parentpath = GetSetting("Folder", "/")
                    .EnableFileOpen = True
                    .EnableFileUpload = True
                    .EnableFileSave = False
                    .ImageSource = "F"
                    .Filename = GetSetting("InitialFilename", "")
                    .CaptureKeys = True
                    .PreviewSize = ParsePreviewSize(rblPreviewSize.SelectedValue)
                    .Caption = "WESNet Image Editor Demo"
                    If cbAspectRatio.Checked Then
                        .AspectRatio = ParseSize(tbRatioX.Text, tbRatioY.Text)
                    End If
                    If cbMinCropArea.Checked Then
                        .MinCropSize = ParseSize(tbMinWidth.Text, tbMinHeight.Text)
                    End If
                    If cbMaxCropArea.Checked Then
                        .MaxCropSize = ParseSize(tbMaxWidth.Text, tbMaxHeight.Text)
                    End If
                End With
                BindFiles()
            End If
        End Sub

        Public Sub ImageEditor1_Cropped(ByVal sender As Object, ByVal e As WESNet.AJAX.Controls.ImageEditor.CropEventArgs) Handles ImageEditor1.Cropped
            Dim CropRectangle As Rectangle = e.CropRectangle
        End Sub

        Protected Sub ImageEditor_LoadImage(ByVal source As Object, ByVal e As WESNet.AJAX.Controls.ImageEditor.ImageEditorEventArgs) Handles ImageEditor1.Reload
            Dim ie As WESNet.AJAX.Controls.ImageEditor.ImageEditor = CType(source, WESNet.AJAX.Controls.ImageEditor.ImageEditor)
            Dim Args As String() = e.Commands.Split("|"c)
            Dim img As System.Drawing.Image
            Select Case Args(0)
                Case "F"
                    If String.IsNullOrEmpty(Args(1)) Then Exit Sub
                    img = GetImageFromFile(Args(1))
                Case "U"
                    If FileUpload1.HasFile Then
                        img = New System.Drawing.Bitmap(FileUpload1.PostedFile.InputStream)
                    Else
                        Exit Sub
                    End If
                Case ""
                    Exit Sub
                Case Else
                    Throw New ArgumentOutOfRangeException("Unknown image load type " & Args(0))
            End Select

            If img Is Nothing Then
                Throw New IOException("Unable to load image from " & Args(0) & " named " & Args(1))
            Else
                ie.Image = img
            End If
        End Sub

        Private Function GetImageFromFile(ByVal Filepath As String) As System.Drawing.Image
            Dim fp As String = Page.Server.MapPath(PortalSettings.HomeDirectory & Filepath)
            If File.Exists(fp) Then
                Return New System.Drawing.Bitmap(fp)
            Else
                Return CreateBlankImage(ImageEditor1.PreviewSize, Color.Black)
            End If
        End Function

        Protected Sub ImageEditor1_SelectFile(ByVal sender As Object, ByVal e As System.EventArgs) Handles ImageEditor1.SelectFile
            trSelectFile.Visible = True
            trSelectUpload.Style.Add("display", "none")
        End Sub

        Protected Sub btnLoad_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles btnLoad.Click
            With ImageEditor1
                .ImageSource = "U"
                .Filename = FileUpload1.FileName
            End With
            trSelectUpload.Style.Add("display", "none")
        End Sub

        Protected Sub ddlFile_SelectedIndexChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles ddlFile.SelectedIndexChanged
            If ddlFile.SelectedIndex > 0 Then
                With ImageEditor1
                    .ImageSource = "F"
                    .Filename = ddlFile.SelectedItem.Text
                End With
                trSelectFile.Visible = False
            End If
        End Sub

        Protected Sub ImageEditor1_SelectUpload(ByVal sender As Object, ByVal e As System.EventArgs) Handles ImageEditor1.SelectUpload
            trSelectFile.Visible = False
            trSelectUpload.Style.Add("display", "")
            btnLoad.Style.Add("display", "none")
        End Sub

        Private Sub Page_PreRender(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.PreRender
            Dim sb As New StringBuilder
            sb.AppendLine("function fileUpload_OnChange (sender, e, btnLoad) {")
            sb.AppendLine("   if (sender.value =='') {")
            sb.AppendLine("        $get(btnLoad).style.display='none';")
            sb.AppendLine("   } else {")
            sb.AppendLine("        $get(btnLoad).style.display='';")
            sb.AppendLine("   }")
            sb.AppendLine("}")
            Page.ClientScript.RegisterClientScriptBlock(Me.GetType, "FileUploadChanged", sb.ToString, True)
        End Sub

        Public ReadOnly Property Files() As ArrayList
            Get
                If _FileList Is Nothing Then
                    _FileList = New ArrayList
                End If
                Return _FileList
            End Get
        End Property

        Private Sub LoadFiles(ByVal Extensions As String, ByVal ExcludeThumbnails As Boolean, ByVal NoneSpecified As Boolean, ByVal Folder As String)
            Files.Clear()
            If ExcludeThumbnails Then
                Dim filesarray As ArrayList = Common.Globals.GetFileList(PortalId, Extensions, NoneSpecified, Folder)
                For Each item As FileItem In filesarray
                    If item.Text.IndexOf(_ThumbSuffix & ".") = -1 Then Files.Add(item)
                Next
            Else
                Files.AddRange(Common.Globals.GetFileList(PortalId, Extensions, NoneSpecified, Folder))
            End If
        End Sub

        Private Sub BindFiles()
            LoadFiles("jpg,gif,png,bmp", True, True, ImageEditor1.Parentpath)
            ddlFile.DataTextField = "Text"
            ddlFile.DataValueField = "Value"
            ddlFile.DataSource = Files
            ddlFile.DataBind()
        End Sub

        Protected Sub btnApplyConstraints_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles btnApplyConstraints.Click
            With ImageEditor1
                .PreviewSize = ParsePreviewSize(rblPreviewSize.SelectedValue)
                .Rescale = cbRescale.Checked
                .ToolbarOrientation = CType([Enum].Parse(GetType(System.Web.UI.WebControls.Orientation), rblOrientation.SelectedValue), System.Web.UI.WebControls.Orientation)
                .CachePersistence = CType([Enum].Parse(GetType(WESNet.AJAX.Controls.ImageEditor.CachePersistenceType), rblCaching.SelectedValue), WESNet.AJAX.Controls.ImageEditor.CachePersistenceType)
                '.UseWebService = cbUseWebService.Checked
                If cbMinCropArea.Checked Then
                    .MinCropSize = ParseSize(tbMinWidth.Text, tbMinHeight.Text)
                Else
                    .MinCropSize = Size.Empty
                End If
                If cbMaxCropArea.Checked Then
                    .MaxCropSize = ParseSize(tbMaxWidth.Text, tbMaxHeight.Text)
                Else
                    .MaxCropSize = Size.Empty
                End If
                If cbAspectRatio.Checked Then
                    .AspectRatio = ParseSize(tbRatioX.Text, tbRatioY.Text)
                Else
                    .AspectRatio = Size.Empty
                End If
                .ShowCropperOnInit = cbShowCropperOnInit.Checked
            End With
        End Sub

        Private Function ParseSize(ByVal XText As String, ByVal YText As String) As Size
            Dim X, Y As Integer
            If Not String.IsNullOrEmpty(XText.Trim()) Then
                X = Integer.Parse(XText.Trim())
            End If
            If Not String.IsNullOrEmpty(YText.Trim()) Then
                Y = Integer.Parse(YText.Trim())
            End If
            If (X = 0 And Y = 0) Then
                Return Size.Empty
            Else
                Return New Size(X, Y)
            End If
        End Function

        Private Function ParsePreviewSize(ByVal WbyH As String) As Size
            Dim value As String = WbyH.Trim()
            If Not String.IsNullOrEmpty(value) Then
                Dim m As RegularExpressions.Match = Regex.Match(value, "^(?<x>\d{1,4})\s*x\s*(?<y>\d{1,4})$")
                If m.Success Then
                    Return ParseSize(m.Groups(1).Value, m.Groups(2).Value)
                End If
            End If
            Return New Size(400, 300)
        End Function

        Private Function GetSetting(Of T)(ByVal key As String, ByVal defaultVaue As T) As T
            Dim obj As Object = Settings(key)
            If obj Is Nothing OrElse CStr(key) = String.Empty Then
                Return defaultVaue
            Else
                If TypeOf defaultVaue Is System.Enum Then
                    Return CType([Enum].Parse(GetType(T), CStr(obj)), T)
                Else
                    Return CType(Settings(key), T)
                End If
            End If
        End Function

        'Creates a blank image (Format24bppRgb pixel format) having the size and color specified
        Private Function CreateBlankImage(ByVal sz As Size, ByVal Color As Color) As Bitmap
            Dim img As Bitmap

            Dim g As Graphics
            Try
                img = New Bitmap(sz.Width, sz.Height, Imaging.PixelFormat.Format24bppRgb)
                g = Graphics.FromImage(img)
                g.Clear(Color)
            Finally
                If Not g Is Nothing Then g.Dispose()
            End Try
            Return img
        End Function

    End Class

End Namespace
